/*
 * Copyright 2000-2010 by Castify Networks
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Castify Networks ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Castify Networks.
 *
 */

package com.aspera.samples.central.query;

import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.axis2.databinding.types.UnsignedInt;

import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferFilter;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferInfo;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferInfoResult;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferStatistics;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferStatisticsResult;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.InfoResult;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionFilter;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionInfo;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionInfoResult;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionStatistics;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionStatisticsResult;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.StatisticsResult;

/**
 * Helper class to log query information to stdout.
 * <p/>
 * 
 * <p/>
 * Copyright 2010 by Aspera, Inc.
 * 
 * @author Aspera
 */
public class QueryLogger {

  private PrintWriter writer = new PrintWriter(System.out, true);
  private long start;
  private long end;

  private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(
      "yyyy-MM-dd HH:mm:ss.sss");

  /**
   * Logs the start of a new query.
   * 
   * @param queryType the query type.
   * @param filter the session filter.
   */
  public void newQuery(QueryType queryType, SessionFilter filter) {
    writer.print("Executing get" + queryType + "( ");
    print(filter);
    writer.println(")");
    start = System.currentTimeMillis();
  }

  /**
   * Logs the start of a new query.
   * 
   * @param queryType the query type.
   * @param filter the file transfer filter.
   */
  public void newQuery(QueryType queryType, FileTransferFilter filter) {
    writer.print("Executing get" + queryType + "( ");
    print(filter);
    writer.println(")");
    start = System.currentTimeMillis();
  }

  /**
   * Logs the given session statistics result.
   * 
   * @param result the result to print.
   * @return the iteration token from the result.
   */
  public String response(SessionStatisticsResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs the given session info result.
   * 
   * @param result the result to print.
   * @return the iteration token.
   */
  public String response(SessionInfoResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs the given file transfer statistics result.
   * 
   * @param result the result to print.
   * @return the iteration token.
   */
  public String response(FileTransferStatisticsResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs the given file transfer info result.
   * 
   * @param result the result to print.
   * @return the iteration token.
   */
  public String response(FileTransferInfoResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs the given info result.
   * 
   * @param result the result to print.
   * @return the iteration token.
   */
  public String response(InfoResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs the given statistics result.
   * 
   * @param result the result to print.
   * @return the iteration token.
   */
  public String response(StatisticsResult result) {
    end = System.currentTimeMillis();
    return print(result);
  }

  /**
   * Logs an error.
   * 
   * @param ex the exception that occured.
   */
  public void error(Exception ex) {
    writer.println("Query call failed:");
    ex.printStackTrace(writer);
    writer.println();
    writer.flush();
  }

  /**
   * Prints the execution time of the query and flushes the writer.
   */
  private void printTrailer() {
    writer.println("Execution time: " + (end - start) + "ms");
    writer.println();
    writer.flush();
  }

  private void print(SessionFilter filter) {
    print("ClientAddress", filter.getClientAddress());
    print("Cookie", filter.getCookie());
    print("CookieFilter", filter.getCookieFilter());
    print("Direction", filter.getDirection());
    print("IterationToken", filter.getIterationToken());
    print("JobId", filter.getJobId());
    print("LastSessionOnly", filter.getLastSessionOnly());
    print("MaxResults", filter.getMaxResults());
    print("MaxSessionEndDate", filter.getMaxSessionEndDate());
    print("MaxSessionStartDate", filter.getMaxSessionStartDate());
    print("MinSessionEndDate", filter.getMinSessionEndDate());
    print("MinSessionStartDate", filter.getMinSessionStartDate());
    print("PrivateKey", filter.getPrivateKey());
    print("ServerAddress", filter.getServerAddress());
    print("SessionStatus", filter.getSessionStatus());
    print("SessionUUID", filter.getSessionUUID());
  }

  private void print(FileTransferFilter filter) {
    print((SessionFilter) filter);
    print("FilenameFilter", filter.getFilenameFilter());
    print("Filepath", filter.getFilepath());
    print("MaxTransferEndDate", filter.getMaxTransferEndDate());
    print("MaxTransferStartDate", filter.getMaxTransferStartDate());
    print("MinTransferEndDate", filter.getMinTransferEndDate());
    print("MinTransferStartDate", filter.getMinTransferStartDate());
    print("TransferId", filter.getTransferId());
    print("TransferStatus", filter.getTransferStatus());
  }

  private void print(FileTransferInfo info) {
    print((FileTransferStatistics) info);
    print("StartDate", info.getStartDate());
    print("EndDate", info.getEndDate());
    print("Checksum", info.getChecksum());
    print("ChecksumType", info.getChecksumType());
    print("FileType", info.getFileType());
    print("Action", info.getAction());
    print("ContentProtection", info.getContentProtection());
  }

  private void print(FileTransferInfo[] info) {
    if (info == null)
      return;
    for (FileTransferInfo fileTransferinfo : info) {
      print(fileTransferinfo);
      writer.println();
    }
  }

  private String print(FileTransferInfoResult result) {
    printResult(
        "FileTransferInfo",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getFileTransferInfo());
    printTrailer();
    return result.getIterationToken();
  }

  private void print(FileTransferStatistics statistics) {
    print("FileTransfer SessionUUID", statistics.getSessionUUID());
    print("LocalId", statistics.getLocalId());
    print("Path", statistics.getPath());
    print("UUID", statistics.getUUID());
    print("Size", statistics.getSize());
    print("Status", statistics.getStatus());
    print("BytesContiguous", statistics.getBytesContiguous());
    print("BytesLost", statistics.getBytesLost());
    print("BytesWritten", statistics.getBytesWritten());
    print("Elapsed", statistics.getElapsed());
    print("ErrorCode", statistics.getErrorCode());
    print("ErrorDescription", statistics.getErrorDescription());
    print("StartOffset", statistics.getStartOffset());
  }

  private void print(FileTransferStatistics[] statistics) {
    if (statistics == null)
      return;
    for (FileTransferStatistics stats : statistics) {
      print(stats);
      writer.println();
    }
  }

  private String print(FileTransferStatisticsResult result) {
    printResult(
        "FileTransferStatistics",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getFileTransferStatistics());
    printTrailer();
    return result.getIterationToken();
  }

  private void print(SessionInfo info) {
    print((SessionStatistics) info);
    print("BandwidthCap", info.getBandwidthCap());
    print("ClientAddress", info.getClientAddress());
    print("ClientEndpoint", info.getClientEndpoint());
    print("ClientFASPPort", info.getClientFASPPort());
    print("Cookie", info.getCookie());
    print("DestinationPath", info.getDestinationPath());
    print("Direction", info.getDirection());
    print("EncryptionCipher", info.getEncryptionCipher());
    print("EndDate", info.getEndDate());
    print("JobMaxRetries", info.getJobMaxRetries());
    print("JobRetryCount", info.getJobRetryCount());
    print("MinimumRate", info.getMinimumRate());
    print("PrivateKey", info.getPrivateKey());
    print("RatePolicy", info.getRatePolicy());
    print("ServerAddress", info.getServerAddress());
    print("ServerEndpoint", info.getServerEndpoint());
    print("ServerFASPPort", info.getServerFASPPort());
    print("ServerSSHPort", info.getServerSSHPort());
    print("SourcePath", info.getSourcePath());
    print("StartDate", info.getStartDate());
    print("TargetRate", info.getTargetRate());
    print("Token", info.getToken());
    print("User", info.getUser());
    print("VLinkId", info.getVLinkId());
  }

  private void print(SessionInfo[] info) {
    if (info == null)
      return;
    for (SessionInfo sessionInfo : info) {
      print(sessionInfo);
      writer.println();
    }
  }

  private String print(SessionInfoResult result) {
    printResult(
        "SessionInfo",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getSessionInfo());
    printTrailer();
    return result.getIterationToken();
  }

  private void print(SessionStatistics statistics) {
    print("Session UUID", statistics.getUUID());
    print("JobId", statistics.getJobId());
    print("Status", statistics.getStatus());
    print("BytesLost", statistics.getBytesLost());
    print("BytesTransferred", statistics.getBytesTransferred());
    print("BytesWritten", statistics.getBytesWritten());
    print("Elapsed", statistics.getElapsed());
    print("ErrorCode", statistics.getErrorCode());
    print("ErrorDescription", statistics.getErrorDescription());
    print("ExpectedByteCount", statistics.getExpectedByteCount());
    print("ExpectedDirCount", statistics.getExpectedDirCount());
    print("ExpectedFileCount", statistics.getExpectedFileCount());
    print("FileCount", statistics.getFileCount());
    print("FilesComplete", statistics.getFilesComplete());
    print("FilesFailed", statistics.getFilesFailed());
    print("FilesTransferring", statistics.getFilesTransferring());
    print("NetworkDelay", statistics.getNetworkDelay());
    print("RateInfo", statistics.getRateInfo());
  }

  private void print(SessionStatistics[] statistics) {
    if (statistics == null)
      return;
    for (SessionStatistics stats : statistics) {
      print(stats);
      writer.println();
    }
  }

  private String print(SessionStatisticsResult result) {
    printResult(
        "SessionStatistics",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getSessionStatistics());
    printTrailer();
    return result.getIterationToken();
  }

  private String print(InfoResult result) {
    printResult(
        "Info",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getSessionInfo());
    writer.println();
    print(result.getFileTransferInfo());
    printTrailer();
    return result.getIterationToken();
  }

  private String print(StatisticsResult result) {
    printResult(
        "Statistics",
        result.getResultCount(),
        result.getRemainingResultCount(),
        result.getIterationToken());
    print(result.getSessionStatistics());
    writer.println();
    print(result.getFileTransferStatistics());
    printTrailer();
    return result.getIterationToken();
  }

  /**
   * Prints a label and a value to the given writer.
   * 
   * @param writer the writer.
   * @param label the label.
   * @param value the value.
   */
  private void print(String label, Object value) {
    if (value == null)
      return;
    if (value instanceof String[]) {
      StringBuilder sb = new StringBuilder();
      for (String val : (String[]) value) {
        if (sb.length() > 0)
          sb.append(", ");
        sb.append(val);
      }
      value = sb;
    } else if (value instanceof Calendar) {
      value = format((Calendar) value);
    }
    String val = String.valueOf(value);
    if (val.length() == 0)
      return;
    writer.print(label + " [" + val + "] ");
  }

  /**
   * Prints the result summary.
   * 
   * @param writer the writer.
   * @param resultsType the result type.
   * @param resultCount the result count.
   * @param remainingResultCount the remaining result count.
   * @param iterationToken the iteration token.
   */
  private void printResult(
      String resultsType,
      UnsignedInt resultCount,
      UnsignedInt remainingResultCount,
      String iterationToken) {

    writer.print(resultsType + " ");
    print("result count", resultCount);
    print("remaining count", remainingResultCount);
    print("iteration token", iterationToken);
    writer.println();

  }

  /**
   * Formats the given calendar object as a string.
   * 
   * @param p_Cal the object to format.
   * @return the formatted string.
   */
  private String format(Calendar p_Cal) {
    return DATE_FORMAT.format(p_Cal.getTime());
  }

}
